Imports Microsoft.VisualBasic
Imports System
Imports System.Collections.Generic
Imports System.ComponentModel
Imports System.Data
Imports System.Drawing
Imports System.Text
Imports System.Windows.Forms

Imports SelectDevice
Imports SMD2XX

Namespace BitBangTest
	Public Partial Class Form1
		Inherits Form
		'
		 '* The BBUSB allows access to 12 pins that can be configured individually
		 '* as either input or output. 8 of these pins are refered to as pins D0 to D7
		 '* while the remaining 4 are referred to as pins C0 to C3 (CBUS pins).
		 '* The D0-D7 pins I/O state is set using the setMode function. The CBUS
		 '* pins must first be set to the I/O mode in EEPROM, then their I/O state
		 '* can also be set using the setMode function. CBUS4 is not configurable for
		 '* I/O.
		 '*
		 '* Select Pin State
		 '* Pin I/O states are selected in the topmost row of buttons.
		 '* Each button is labeled with the corresponding pin name and clicking the
		 '* button will toggle the pin I/O state and show that state as either an 'I'
		 '* or 'O'.
		 '*
		 '* Toggle Output Pin Logic Level
		 '* The second row uses buttons with virtual switch images. If the pins I/O
		 '* state is for output, clicking the button will toggle ON and OFF changing
		 '* the logic level on the pin. If the pin I/O state is for input, the button
		 '* will not be clickable and will appear grayed out.
		 '*
		 '*
		 '* Read Input Pin Logic Level
		 '* Pins with their I/O state set to input will be read and the state will be
		 '* shown in the bottom row by showing a virtual LED as either an OFF or ON image.
		 '* Pins with their I/O state set to output will show a light gray LED image.
		 '*
		 '* Revision History
		 '* Version 1.0 Joe Pardue 6/3/07
		 '*
		 '

		#Region "Intitialization"

		Private ftStatus As FT_STATUS = 0
        Private ftHandle As UInt32 = 0

		Private SM As SM_D2XX = New SM_D2XX()

		Private dev As Select_Device = New Select_Device()

		' Constants for logic levels
		Private Const [ON] As Boolean = True
		Private Const [OFF] As Boolean = False

		Private Input As Byte = 0
		Private Output As Integer = 0
		Private InputC As Byte = 0
		Private OutputC As Integer = &HF0 ' Starts with C pins as output

		Private UseDBus As Boolean = True ' If false, use CBUS

		' Constants to indicate pin state
		Private Const State_In As Boolean = True
		Private Const State_Out As Boolean = False

		' Pin State Variables
		Private D0_IO_State As Boolean = State_Out
		Private D0_OnOff_State As Boolean = [OFF]

		Private D1_IO_State As Boolean = State_Out
		Private D1_OnOff_State As Boolean = [OFF]

		Private D2_IO_State As Boolean = State_Out
		Private D2_OnOff_State As Boolean = [OFF]

		Private D3_IO_State As Boolean = State_Out
		Private D3_OnOff_State As Boolean = [OFF]

		Private D4_IO_State As Boolean = State_Out
		Private D4_OnOff_State As Boolean = [OFF]

		Private D5_IO_State As Boolean = State_Out
		Private D5_OnOff_State As Boolean = [OFF]

		Private D6_IO_State As Boolean = State_Out
		Private D6_OnOff_State As Boolean = [OFF]

		Private D7_IO_State As Boolean = State_Out
		Private D7_OnOff_State As Boolean = [OFF]

		Private C0_IO_State As Boolean = State_Out
		Private C0_OnOff_State As Boolean = [OFF]

		Private C1_IO_State As Boolean = State_Out
		Private C1_OnOff_State As Boolean = [OFF]

		Private C2_IO_State As Boolean = State_Out
		Private C2_OnOff_State As Boolean = [OFF]

		Private C3_IO_State As Boolean = State_Out
		Private C3_OnOff_State As Boolean = [OFF]

		Public Sub New()
			InitializeComponent()

			LED0.image = CInt(Fix(LED.LED22.LEDColor.Disabled))
			buttonSD0.ImageIndex = 0
			buttonSD0.Show()

			LED1.image = CInt(Fix(LED.LED22.LEDColor.Disabled))
			buttonSD1.ImageIndex = 0
			buttonSD1.Show()

			LED2.image = CInt(Fix(LED.LED22.LEDColor.Disabled))
			buttonSD2.ImageIndex = 0
			buttonSD2.Show()

			LED3.image = CInt(Fix(LED.LED22.LEDColor.Disabled))
			buttonSD3.ImageIndex = 0
			buttonSD3.Show()

			LED4.image = CInt(Fix(LED.LED22.LEDColor.Disabled))
			buttonSD4.ImageIndex = 0
			buttonSD4.Show()

			LED5.image = CInt(Fix(LED.LED22.LEDColor.Disabled))
			buttonSD5.ImageIndex = 0
			buttonSD5.Show()

			LED6.image = CInt(Fix(LED.LED22.LEDColor.Disabled))
			buttonSD6.ImageIndex = 0
			buttonSD6.Show()

			LED7.image = CInt(Fix(LED.LED22.LEDColor.Disabled))
			buttonSD7.ImageIndex = 0
			buttonSD7.Show()

			LEDC0.image = CInt(Fix(LED.LED22.LEDColor.Disabled))
			buttonSC0.ImageIndex = 0
			buttonSC0.Show()

			LEDC1.image = CInt(Fix(LED.LED22.LEDColor.Disabled))
			buttonSC1.ImageIndex = 0
			buttonSC1.Show()

			LEDC2.image = CInt(Fix(LED.LED22.LEDColor.Disabled))
			buttonSC2.ImageIndex = 0
			buttonSC2.Show()

			LEDC3.image = CInt(Fix(LED.LED22.LEDColor.Disabled))
			buttonSC3.ImageIndex = 0
			buttonSC3.Show()

			groupBoxDBUS.Enabled = True
			groupBoxCBUS.Enabled = False

		End Sub
		#End Region

		#Region "Menu Items"

		#Region "Bus select toggle"
		Private Sub useDBusToolStripMenuItem_Click(ByVal sender As Object, ByVal e As EventArgs) Handles useDBusToolStripMenuItem.Click
			If UseDBus Then
				UseDBus = False ' Use C BUS
				useDBusToolStripMenuItem.Text = "Use C BUS"
				groupBoxDBUS.Enabled = False
				groupBoxCBUS.Enabled = True
			Else
				UseDBus = True ' Use D BUS
				useDBusToolStripMenuItem.Text = "Use D BUS"
				groupBoxDBUS.Enabled = True
				groupBoxCBUS.Enabled = False
			End If
		End Sub
		#End Region

		#Region "Select and Open a Device"

		Private Sub selectDeviceToolStripMenuItem_Click(ByVal sender As Object, ByVal e As EventArgs) Handles selectDeviceToolStripMenuItem.Click
			dev.ShowDialog()

			Select Case dev.OpenBy
                Case CUInt(OPEN_BY.SERIAL_NUMBER) ' use OpenEX
                    openBySerialNumber(dev.SerialNumber)
                Case CUInt(OPEN_BY.DESCRIPTION) ' use OpenEX
                    openByDescription(dev.Description)
                Case CUInt(OPEN_BY.LOCATIONID) ' use OpenEX
                    openByLocationID(dev.LocID)
				Case Else
					MessageBox.Show("ERROR - OpenBy = " & dev.OpenBy.ToString())
			End Select
		End Sub

		' open by serial number
		Private Sub openBySerialNumber(ByVal SN As String)
            ftStatus = SM.openSerialNumber(SN, ftHandle)
			If ftStatus <> FT_STATUS.FT_OK Then
				MessageBox.Show("Error: buttonGet_Click failed to open device")
			End If
		End Sub

		' open by description
		Private Sub openByDescription(ByVal description As String)
            ftStatus = SM.openDescription(description, ftHandle)
			If ftStatus <> FT_STATUS.FT_OK Then
				MessageBox.Show("Error: buttonGet_Click failed to open port")
			End If
		End Sub

		' open by location
		Private Sub openByLocationID(ByVal locID As UInt32)
            ftStatus = SM.openLocationID(locID, ftHandle)
			If ftStatus <> FT_STATUS.FT_OK Then
				MessageBox.Show("Error: buttonGet_Click failed to open port")
			End If
		End Sub

		' Assumes device is open
		Private Sub closeDevice()
            SM.closeDevice(ftHandle)
		End Sub

		#End Region

		#End Region

		#Region "Set I/O State"

		Private Sub buttonD0_Click(ByVal sender As Object, ByVal e As EventArgs) Handles buttonD0.Click
			If D0_IO_State = State_In Then
				D0_IO_State = State_Out
				buttonD0.Text = "Out"
				buttonSD0.Enabled = True
				LED0.image = CInt(Fix(LED.LED22.LEDColor.Disabled))
				If getOutputBit0() Then
					labelD0.Text = "1"
				Else
						labelD0.Text = "0"
				End If
			Else
				D0_IO_State = State_In
				buttonD0.Text = "In"
				buttonSD0.Enabled = False
				LED0.image = CInt(Fix(LED.LED22.LEDColor.RedOff))
				labelD0.Text = "X"
			End If
		End Sub
		Private Sub buttonD1_Click(ByVal sender As Object, ByVal e As EventArgs) Handles buttonD1.Click
			If D1_IO_State = State_In Then
				D1_IO_State = State_Out
				buttonD1.Text = "Out"
				buttonSD1.Enabled = True
				LED1.image = CInt(Fix(LED.LED22.LEDColor.Disabled))
				If getOutputBit1() Then
					labelD1.Text = "1"
				Else
						labelD1.Text = "0"
				End If
			Else
				D1_IO_State = State_In
				buttonD1.Text = "In"
				buttonSD1.Enabled = False
				LED1.image = CInt(Fix(LED.LED22.LEDColor.RedOff))
				labelD1.Text = "X"
			End If
		End Sub
		Private Sub buttonD2_Click(ByVal sender As Object, ByVal e As EventArgs) Handles buttonD2.Click
			If D2_IO_State = State_In Then
				D2_IO_State = State_Out
				buttonD2.Text = "Out"
				buttonSD2.Enabled = True
				LED2.image = CInt(Fix(LED.LED22.LEDColor.Disabled))
				If getOutputBit2() Then
					labelD2.Text = "1"
				Else
						labelD2.Text = "0"
				End If
			Else
				D2_IO_State = State_In
				buttonD2.Text = "In"
				buttonSD2.Enabled = False
				LED2.image = CInt(Fix(LED.LED22.LEDColor.RedOff))
				labelD2.Text = "X"
			End If
		End Sub

		Private Sub buttonD3_Click(ByVal sender As Object, ByVal e As EventArgs) Handles buttonD3.Click
			If D3_IO_State = State_In Then
				D3_IO_State = State_Out
				buttonD3.Text = "Out"
				buttonSD3.Enabled = True
				LED3.image = CInt(Fix(LED.LED22.LEDColor.Disabled))
				If getOutputBit3() Then
					labelD3.Text = "1"
				Else
						labelD3.Text = "0"
				End If
			Else
				D3_IO_State = State_In
				buttonD3.Text = "In"
				buttonSD3.Enabled = False
				LED3.image = CInt(Fix(LED.LED22.LEDColor.RedOff))
				labelD3.Text = "X"
			End If
		End Sub

		Private Sub buttonD4_Click(ByVal sender As Object, ByVal e As EventArgs) Handles buttonD4.Click
			If D4_IO_State = State_In Then
				D4_IO_State = State_Out
				buttonD4.Text = "Out"
				buttonSD4.Enabled = True
				LED4.image = CInt(Fix(LED.LED22.LEDColor.Disabled))
				If getOutputBit4() Then
					labelD4.Text = "1"
				Else
						labelD4.Text = "0"
				End If
			Else
				D4_IO_State = State_In
				buttonD4.Text = "In"
				buttonSD4.Enabled = False
				LED4.image = CInt(Fix(LED.LED22.LEDColor.RedOff))
				labelD4.Text = "X"
			End If
		End Sub

		Private Sub buttonD5_Click(ByVal sender As Object, ByVal e As EventArgs) Handles buttonD5.Click
			If D5_IO_State = State_In Then
				D5_IO_State = State_Out
				buttonD5.Text = "Out"
				buttonSD5.Enabled = True
				LED5.image = CInt(Fix(LED.LED22.LEDColor.Disabled))
				If getOutputBit5() Then
					labelD5.Text = "1"
				Else
						labelD5.Text = "0"
				End If
			Else
				D5_IO_State = State_In
				buttonD5.Text = "In"
				buttonSD5.Enabled = False
				LED5.image = CInt(Fix(LED.LED22.LEDColor.RedOff))
				labelD5.Text = "X"
			End If
		End Sub

		Private Sub buttonD6_Click(ByVal sender As Object, ByVal e As EventArgs) Handles buttonD6.Click
			If D6_IO_State = State_In Then
				D6_IO_State = State_Out
				buttonD6.Text = "Out"
				buttonSD6.Enabled = True
				LED6.image = CInt(Fix(LED.LED22.LEDColor.Disabled))
				If getOutputBit6() Then
					labelD6.Text = "1"
				Else
						labelD6.Text = "0"
				End If
			Else
				D6_IO_State = State_In
				buttonD6.Text = "In"
				buttonSD6.Enabled = False
				LED6.image = CInt(Fix(LED.LED22.LEDColor.RedOff))
				labelD6.Text = "X"
			End If
		End Sub

		Private Sub buttonD7_Click(ByVal sender As Object, ByVal e As EventArgs) Handles buttonD7.Click
			If D7_IO_State = State_In Then
				D7_IO_State = State_Out
				buttonD7.Text = "Out"
				buttonSD7.Enabled = True
				LED7.image = CInt(Fix(LED.LED22.LEDColor.Disabled))
				If getOutputBit1() Then
					labelD7.Text = "1"
				Else
						labelD7.Text = "0"
				End If
			Else
				D7_IO_State = State_In
				buttonD7.Text = "In"
				buttonSD7.Enabled = False
				LED7.image = CInt(Fix(LED.LED22.LEDColor.RedOff))
				labelD7.Text = "X"
			End If
		End Sub

		Private Sub buttonC0_Click(ByVal sender As Object, ByVal e As EventArgs) Handles buttonC0.Click
			If C0_IO_State = State_In Then
				C0_IO_State = State_Out
				buttonC0.Text = "Out"
				buttonSC0.Enabled = True
				LEDC0.image = CInt(Fix(LED.LED22.LEDColor.Disabled))
			Else
				C0_IO_State = State_In
				buttonC0.Text = "In"
				buttonSC0.Enabled = False
				LEDC0.image = CInt(Fix(LED.LED22.LEDColor.RedOff))
				labelC0.Text = "X"
			End If
		End Sub

		Private Sub buttonC1_Click(ByVal sender As Object, ByVal e As EventArgs) Handles buttonC1.Click
			If C1_IO_State = State_In Then
				C1_IO_State = State_Out
				buttonC1.Text = "Out"
				buttonSC1.Enabled = True
				LEDC1.image = CInt(Fix(LED.LED22.LEDColor.Disabled))
			Else
				C1_IO_State = State_In
				buttonC1.Text = "In"
				buttonSC1.Enabled = False
				LEDC1.image = CInt(Fix(LED.LED22.LEDColor.RedOff))
				labelC1.Text = "X"
			End If
		End Sub

		Private Sub buttonC2_Click(ByVal sender As Object, ByVal e As EventArgs) Handles buttonC2.Click
			If C2_IO_State = State_In Then
				C2_IO_State = State_Out
				buttonC2.Text = "Out"
				buttonSC2.Enabled = True
				LEDC2.image = CInt(Fix(LED.LED22.LEDColor.Disabled))
				If getOutputBitC2() Then
					labelC2.Text = "1"
				Else
						labelC2.Text = "0"
				End If
			Else
				C2_IO_State = State_In
				buttonC2.Text = "In"
				buttonSC2.Enabled = False
				LEDC2.image = CInt(Fix(LED.LED22.LEDColor.RedOff))
				labelC2.Text = "X"
			End If
		End Sub

		Private Sub buttonC3_Click(ByVal sender As Object, ByVal e As EventArgs) Handles buttonC3.Click
			If C3_IO_State = State_In Then
				C3_IO_State = State_Out
				buttonC3.Text = "Out"
				buttonSC3.Enabled = True
				LEDC3.image = CInt(Fix(LED.LED22.LEDColor.Disabled))
			Else
				C3_IO_State = State_In
				buttonC3.Text = "In"
				buttonSC3.Enabled = False
				LEDC3.image = CInt(Fix(LED.LED22.LEDColor.RedOff))
				labelC3.Text = "X"
			End If
		End Sub

		#End Region

		#Region "Toggle Output Logic Levels"

		Private Sub buttonSD0_Click(ByVal sender As Object, ByVal e As EventArgs) Handles buttonSD0.Click
			If D0_IO_State = State_Out Then
				If D0_OnOff_State = [ON] Then
					D0_OnOff_State = [OFF]
					buttonSD0.ImageIndex = 0
					labelD0.Text = "0"
					If Output >= 1 Then
					Output -= 1
					End If
				Else
					D0_OnOff_State = [ON]
					buttonSD0.ImageIndex = 1
					labelD0.Text = "1"
					Output += 1
				End If
				bitBangOutD(CByte(Output))
			End If
		End Sub

		Private Sub buttonSD1_Click(ByVal sender As Object, ByVal e As EventArgs) Handles buttonSD1.Click
			If D1_IO_State = State_Out Then
				If D1_OnOff_State = [ON] Then
					D1_OnOff_State = [OFF]
					buttonSD1.ImageIndex = 0
					labelD1.Text = "0"
					If Output >= 2 Then
					Output -= 2
					End If
				Else
					D1_OnOff_State = [ON]
					buttonSD1.ImageIndex = 1
					labelD1.Text = "1"
					Output += 2
				End If
				bitBangOutD(CByte(Output))
			End If
		End Sub

		Private Sub buttonSD2_Click(ByVal sender As Object, ByVal e As EventArgs) Handles buttonSD2.Click
			If D2_IO_State = State_Out Then
				If D2_OnOff_State = [ON] Then
					D2_OnOff_State = [OFF]
					buttonSD2.ImageIndex = 0
					labelD2.Text = "0"
					If Output >= 4 Then
					Output -= 4
					End If
				Else
					D2_OnOff_State = [ON]
					buttonSD2.ImageIndex = 1
					labelD2.Text = "1"
					Output += 4
				End If
				bitBangOutD(CByte(Output))
			End If
		End Sub
		Private Sub buttonSD3_Click(ByVal sender As Object, ByVal e As EventArgs) Handles buttonSD3.Click
			If D3_IO_State = State_Out Then
				If D3_OnOff_State = [ON] Then
					D3_OnOff_State = [OFF]
					buttonSD3.ImageIndex = 0
					labelD3.Text = "0"
					If Output >= 8 Then
					Output -= 8
					End If
				Else
					D3_OnOff_State = [ON]
					buttonSD3.ImageIndex = 1
					labelD3.Text = "1"
					Output += 8
				End If
				bitBangOutD(CByte(Output))
			End If
		End Sub

		Private Sub buttonSD4_Click(ByVal sender As Object, ByVal e As EventArgs) Handles buttonSD4.Click
			If D4_IO_State = State_Out Then
				If D4_OnOff_State = [ON] Then
					D4_OnOff_State = [OFF]
					buttonSD4.ImageIndex = 0
					labelD4.Text = "0"
					If Output >= 16 Then
					Output -= 16
					End If
				Else
					D4_OnOff_State = [ON]
					buttonSD4.ImageIndex = 1
					labelD4.Text = "1"
					Output += 16
				End If
				bitBangOutD(CByte(Output))
			End If
		End Sub

		Private Sub buttonSD5_Click(ByVal sender As Object, ByVal e As EventArgs) Handles buttonSD5.Click
			If D5_IO_State = State_Out Then
				If D5_OnOff_State = [ON] Then
					D5_OnOff_State = [OFF]
					buttonSD5.ImageIndex = 0
					labelD5.Text = "0"
					If Output >= 32 Then
					Output -= 32
					End If
				Else
					D5_OnOff_State = [ON]
					buttonSD5.ImageIndex = 1
					labelD5.Text = "1"
					Output += 32
				End If
				bitBangOutD(CByte(Output))
			End If
		End Sub

		Private Sub buttonSD6_Click(ByVal sender As Object, ByVal e As EventArgs) Handles buttonSD6.Click
			If D6_IO_State = State_Out Then
				If D6_OnOff_State = [ON] Then
					D6_OnOff_State = [OFF]
					buttonSD6.ImageIndex = 0
					labelD6.Text = "0"
					If Output >= 64 Then
					Output -= 64
					End If
				Else
					D6_OnOff_State = [ON]
					buttonSD6.ImageIndex = 1
					labelD6.Text = "1"
					Output += 64
				End If
				bitBangOutD(CByte(Output))
			End If
		End Sub

		Private Sub buttonSD7_Click(ByVal sender As Object, ByVal e As EventArgs) Handles buttonSD7.Click
			If D7_IO_State = State_Out Then
				If D7_OnOff_State = [ON] Then
					D7_OnOff_State = [OFF]
					buttonSD7.ImageIndex = 0
					labelD7.Text = "0"
					If Output >= 128 Then
					Output -= 128
					End If
				Else
					D7_OnOff_State = [ON]
					buttonSD7.ImageIndex = 1
					labelD7.Text = "1"
					Output += 128
				End If
				bitBangOutD(CByte(Output))
			End If
		End Sub

		Private Sub buttonSC0_Click(ByVal sender As Object, ByVal e As EventArgs) Handles buttonSC0.Click
			If C0_IO_State = State_Out Then
				If C0_OnOff_State = [ON] Then
					C0_OnOff_State = [OFF]
					buttonSC0.ImageIndex = 0
					labelC0.Text = "0"
					If OutputC >= 1 Then
					OutputC -= 1
					End If
				Else
					C0_OnOff_State = [ON]
					buttonSC0.ImageIndex = 1
					labelC0.Text = "1"
					OutputC += 1
				End If
				bitBangOutC(CByte(OutputC))
			End If
		End Sub

		Private Sub buttonSC1_Click(ByVal sender As Object, ByVal e As EventArgs) Handles buttonSC1.Click
			If C1_IO_State = State_Out Then
				If C1_OnOff_State = [ON] Then
					C1_OnOff_State = [OFF]
					buttonSC1.ImageIndex = 0
					labelC1.Text = "0"
					If OutputC >= 2 Then
					OutputC -= 2
					End If
				Else
					C1_OnOff_State = [ON]
					buttonSC1.ImageIndex = 1
					labelC1.Text = "1"
					OutputC += 2
				End If
				bitBangOutC(CByte(OutputC))
			End If
		End Sub

		Private Sub buttonSC2_Click(ByVal sender As Object, ByVal e As EventArgs) Handles buttonSC2.Click
			If C2_IO_State = State_Out Then
				If C2_OnOff_State = [ON] Then
					C2_OnOff_State = [OFF]
					buttonSC2.ImageIndex = 0
					labelC2.Text = "0"
					If OutputC >= 4 Then
					OutputC -= 4
					End If
				Else
					C2_OnOff_State = [ON]
					buttonSC2.ImageIndex = 1
					labelC2.Text = "1"
					OutputC += 4
				End If
				bitBangOutC(CByte(OutputC))
			End If
		End Sub

		Private Sub buttonSC3_Click(ByVal sender As Object, ByVal e As EventArgs) Handles buttonSC3.Click
			If C3_IO_State = State_Out Then
				If C3_OnOff_State = [ON] Then
					C3_OnOff_State = [OFF]
					buttonSC3.ImageIndex = 0
					labelC3.Text = "0"
					If OutputC >= 8 Then
					OutputC -= 8
					End If
				Else
					C3_OnOff_State = [ON]
					buttonSC3.ImageIndex = 1
					labelC3.Text = "1"
					OutputC += 8
				End If
				bitBangOutC(CByte(OutputC))
			End If
		End Sub
		#End Region

		#Region "Get Input Bits"

		Private Function getInputBit0() As Boolean
			Dim bit0 As Byte = CByte(Input)

			bit0 = CByte(bit0 << 7)
			bit0 = CByte(bit0 >> 7)

			If bit0 = 1 Then
			Return True
			Else
				Return False
			End If
		End Function
		Private Function getInputBit1() As Boolean
			Dim bit1 As Byte = CByte(Input)

			bit1 = CByte(bit1 << 6)
			bit1 = CByte(bit1 >> 7)

			If bit1 = 1 Then
			Return True
			Else
				Return False
			End If
		End Function
		Private Function getInputBit2() As Boolean
			Dim bit2 As Byte = CByte(Input)

			bit2 = CByte(bit2 << 5)
			bit2 = CByte(bit2 >> 7)

			If bit2 = 1 Then
			Return True
			Else
				Return False
			End If
		End Function

		Private Function getInputBit3() As Boolean
			Dim bit3 As Byte = CByte(Input)

			bit3 = CByte(bit3 << 4)
			bit3 = CByte(bit3 >> 7)

			If bit3 = 1 Then
			Return True
			Else
				Return False
			End If
		End Function

		Private Function getInputBit4() As Boolean
			Dim bit4 As Byte = CByte(Input)

			bit4 = CByte(bit4 << 3)
			bit4 = CByte(bit4 >> 7)

			If bit4 = 1 Then
			Return True
			Else
				Return False
			End If
		End Function

		Private Function getInputBit5() As Boolean
			Dim bit5 As Byte = CByte(Input)

			bit5 = CByte(bit5 << 2)
			bit5 = CByte(bit5 >> 7)

			If bit5 = 1 Then
			Return True
			Else
				Return False
			End If
		End Function

		Private Function getInputBit6() As Boolean
			Dim bit6 As Byte = CByte(Input)

			bit6 = CByte(bit6 << 1)
			bit6 = CByte(bit6 >> 7)

			If bit6 = 1 Then
			Return True
			Else
				Return False
			End If
		End Function

		Private Function getInputBit7() As Boolean
			Dim bit7 As Byte = CByte(Input)

			bit7 = CByte(bit7 >> 7)

			If bit7 = 1 Then
			Return True
			Else
				Return False
			End If
		End Function

		Private Function getInputBitC0() As Boolean
			Dim bit0 As Byte = CByte(InputC)

			bit0 = CByte(bit0 << 7)
			bit0 = CByte(bit0 >> 7)

			If bit0 = 1 Then
			Return True
			Else
				Return False
			End If
		End Function
		Private Function getInputBitC1() As Boolean
			Dim bit1 As Byte = CByte(InputC)

			bit1 = CByte(bit1 << 6)
			bit1 = CByte(bit1 >> 7)

			If bit1 = 1 Then
			Return True
			Else
				Return False
			End If
		End Function
		Private Function getInputBitC2() As Boolean
			Dim bit2 As Byte = CByte(InputC)

			bit2 = CByte(bit2 << 5)
			bit2 = CByte(bit2 >> 7)

			If bit2 = 1 Then
			Return True
			Else
				Return False
			End If
		End Function

		Private Function getInputBitC3() As Boolean
			Dim bit3 As Byte = CByte(InputC)

			bit3 = CByte(bit3 << 4)
			bit3 = CByte(bit3 >> 7)

			If bit3 = 1 Then
			Return True
			Else
				Return False
			End If
		End Function
		#End Region

		#Region "Get Output Bits"

		Private Function getOutputBit0() As Boolean
			Dim bit0 As Byte = CByte(Output)

			bit0 = CByte(bit0 << 7)
			bit0 = CByte(bit0 >> 7)

			If bit0 = 1 Then
			Return True
			Else
				Return False
			End If
		End Function
		Private Function getOutputBit1() As Boolean
			Dim bit1 As Byte = CByte(Output)

			bit1 = CByte(bit1 << 6)
			bit1 = CByte(bit1 >> 7)

			If bit1 = 1 Then
			Return True
			Else
				Return False
			End If
		End Function
		Private Function getOutputBit2() As Boolean
			Dim bit2 As Byte = CByte(Output)

			bit2 = CByte(bit2 << 5)
			bit2 = CByte(bit2 >> 7)

			If bit2 = 1 Then
			Return True
			Else
				Return False
			End If
		End Function

		Private Function getOutputBit3() As Boolean
			Dim bit3 As Byte = CByte(Output)

			bit3 = CByte(bit3 << 4)
			bit3 = CByte(bit3 >> 7)

			If bit3 = 1 Then
			Return True
			Else
				Return False
			End If
		End Function

		Private Function getOutputBit4() As Boolean
			Dim bit4 As Byte = CByte(Output)

			bit4 = CByte(bit4 << 3)
			bit4 = CByte(bit4 >> 7)

			If bit4 = 1 Then
			Return True
			Else
				Return False
			End If
		End Function

		Private Function getOutputBit5() As Boolean
			Dim bit5 As Byte = CByte(Output)

			bit5 = CByte(bit5 << 2)
			bit5 = CByte(bit5 >> 7)

			If bit5 = 1 Then
			Return True
			Else
				Return False
			End If
		End Function

		Private Function getOutputBit6() As Boolean
			Dim bit6 As Byte = CByte(Output)

			bit6 = CByte(bit6 << 1)
			bit6 = CByte(bit6 >> 7)

			If bit6 = 1 Then
			Return True
			Else
				Return False
			End If
		End Function

		Private Function getOutputBit7() As Boolean
			Dim bit7 As Byte = CByte(Output)

			bit7 = CByte(bit7 >> 7)

			If bit7 = 1 Then
			Return True
			Else
				Return False
			End If
		End Function

		Private Function getOutputBitC0() As Boolean
			Dim bit0 As Byte = CByte(OutputC)

			bit0 = CByte(bit0 << 7)
			bit0 = CByte(bit0 >> 7)

			If bit0 = 1 Then
			Return True
			Else
				Return False
			End If
		End Function
		Private Function getOutputBitC1() As Boolean
			Dim bit1 As Byte = CByte(OutputC)

			bit1 = CByte(bit1 << 6)
			bit1 = CByte(bit1 >> 7)

			If bit1 = 1 Then
			Return True
			Else
				Return False
			End If
		End Function
		Private Function getOutputBitC2() As Boolean
			Dim bit2 As Byte = CByte(OutputC)

			bit2 = CByte(bit2 << 5)
			bit2 = CByte(bit2 >> 7)

			If bit2 = 1 Then
			Return True
			Else
				Return False
			End If
		End Function

		Private Function getOutputBitC3() As Boolean
			Dim bit3 As Byte = CByte(OutputC)

			bit3 = CByte(bit3 << 4)
			bit3 = CByte(bit3 >> 7)

			If bit3 = 1 Then
			Return True
			Else
				Return False
			End If
		End Function
		#End Region

		#Region "Read Bus"

		Private Sub buttonReadDBus_Click(ByVal sender As Object, ByVal e As EventArgs) Handles buttonReadDBus.Click
            SM.bitBang_GetBitMode(ftHandle, Input)

			If D0_IO_State = State_In Then
				If getInputBit0() Then
					LED0.image = CInt(Fix(LED.LED22.LEDColor.RedOff))
				Else
						LED0.image = CInt(Fix(LED.LED22.LEDColor.RedOn))
				End If
			End If

			If D1_IO_State = State_In Then
				If getInputBit1() Then
					LED1.image = CInt(Fix(LED.LED22.LEDColor.RedOff))
				Else
						LED1.image = CInt(Fix(LED.LED22.LEDColor.RedOn))
				End If
			End If

			If D2_IO_State = State_In Then
				If getInputBit2() Then
					LED2.image = CInt(Fix(LED.LED22.LEDColor.RedOff))
				Else
						LED2.image = CInt(Fix(LED.LED22.LEDColor.RedOn))
				End If
			End If

			If D3_IO_State = State_In Then
				If getInputBit3() Then
					LED3.image = CInt(Fix(LED.LED22.LEDColor.RedOff))
				Else
						LED3.image = CInt(Fix(LED.LED22.LEDColor.RedOn))
				End If
			End If

			If D4_IO_State = State_In Then
				If getInputBit4() Then
					LED4.image = CInt(Fix(LED.LED22.LEDColor.RedOff))
				Else
						LED4.image = CInt(Fix(LED.LED22.LEDColor.RedOn))
				End If
			End If

			If D5_IO_State = State_In Then
				If getInputBit5() Then
					LED5.image = CInt(Fix(LED.LED22.LEDColor.RedOff))
				Else
						LED5.image = CInt(Fix(LED.LED22.LEDColor.RedOn))
				End If
			End If

			If D6_IO_State = State_In Then
				If getInputBit6() Then
					LED6.image = CInt(Fix(LED.LED22.LEDColor.RedOff))
				Else
						LED6.image = CInt(Fix(LED.LED22.LEDColor.RedOn))
				End If
			End If

			If D7_IO_State = State_In Then
				If getInputBit7() Then
					LED7.image = CInt(Fix(LED.LED22.LEDColor.RedOff))
				Else
						LED7.image = CInt(Fix(LED.LED22.LEDColor.RedOn))
				End If
			End If
		End Sub

		Private Sub buttonReadCBus_Click(ByVal sender As Object, ByVal e As EventArgs) Handles buttonReadCBus.Click
			' Set CBUS pins to I/O state
			Dim temp As Byte = 0

			If C0_IO_State = State_Out Then
                temp += Convert.ToByte(16)
			End If

			If C1_IO_State = State_Out Then
                temp += Convert.ToByte(32)
			End If

			If C0_IO_State = State_Out Then
                temp += Convert.ToByte(64)
			End If

			If C0_IO_State = State_Out Then
                temp += Convert.ToByte(128)
			End If

			' Set to input on CBUS pins
            SM.bitBang_SetBitMode(ftHandle, temp, &H20)

			' Read the input
            SM.bitBang_GetBitMode(ftHandle, InputC)

			If C0_IO_State = State_In Then
				If getInputBitC0() Then
					LEDC0.image = CInt(Fix(LED.LED22.LEDColor.RedOff))
				Else
						LEDC0.image = CInt(Fix(LED.LED22.LEDColor.RedOn))
				End If
			End If

			If C1_IO_State = State_In Then
				If getInputBitC1() Then
					LEDC1.image = CInt(Fix(LED.LED22.LEDColor.RedOff))
				Else
						LEDC1.image = CInt(Fix(LED.LED22.LEDColor.RedOn))
				End If
			End If

			If C2_IO_State = State_In Then
				If getInputBitC2() Then
					LEDC2.image = CInt(Fix(LED.LED22.LEDColor.RedOff))
				Else
						LEDC2.image = CInt(Fix(LED.LED22.LEDColor.RedOn))
				End If
			End If

			If C3_IO_State = State_In Then
				If getInputBitC3() Then
					LEDC3.image = CInt(Fix(LED.LED22.LEDColor.RedOff))
				Else
						LEDC3.image = CInt(Fix(LED.LED22.LEDColor.RedOn))
				End If
			End If
		End Sub

		#End Region

		#Region "Bit-bang Output"

		Private Sub bitBangOutD(ByVal Out As Byte)
            SM.bitBang_SetBitMode(ftHandle, Out, &H1)
		End Sub

		Private Sub bitBangOutC(ByVal Out As Byte)
            SM.bitBang_SetBitMode(ftHandle, Out, &H20)
		End Sub

		#End Region

	End Class
End Namespace

