Imports Microsoft.VisualBasic
Imports System
Imports System.Collections.Generic
Imports System.Text
Imports System.Windows.Forms

' required for DLLImport
Imports System.Runtime.InteropServices

Imports FT_HANDLE = System.UInt32

Namespace SMD2XX

#Region "type definitions"
    ' enum used for the various possible returns from the D2XX function calls
    Public Enum FT_STATUS
        FT_OK = 0
        FT_INVALID_HANDLE
        FT_DEVICE_NOT_FOUND
        FT_DEVICE_NOT_OPENED
        FT_IO_ERROR
        FT_INSUFFICIENT_RESOURCES
        FT_INVALID_PARAMETER
        FT_INVALID_BAUD_RATE
        FT_DEVICE_NOT_OPENED_FOR_ERASE
        FT_DEVICE_NOT_OPENED_FOR_WRITE
        FT_FAILED_TO_WRITE_DEVICE
        FT_EEPROM_READ_FAILED
        FT_EEPROM_WRITE_FAILED
        FT_EEPROM_ERASE_FAILED
        FT_EEPROM_NOT_PRESENT
        FT_EEPROM_NOT_PROGRAMMED
        FT_INVALID_ARGS
        FT_OTHER_ERROR
    End Enum
#End Region

    Friend Class SM_D2XX

#Region "DLL declarations"

#Region "Open and Close"

        <DllImport("FTD2XX.dll")> _
        Shared Function FT_Open(ByVal uiDevice As UInt32, ByRef ftHandle As UInt32) As FT_STATUS
        End Function

        ' Two overload versions for FT_OpenEX
        ' Flags = 1 opens by serial number
        ' Flags = 2 opens by description
        ' Flags = 3 opens by Location Id
        ' needed two overloads so that we can send either a string or a UInt32

        ' open by serial number or description string
        <DllImport("FTD2XX.dll")> _
        Shared Function FT_OpenEx(<MarshalAs(UnmanagedType.LPStr)> ByVal str As String, ByVal Flags As UInt32, ByRef ftHandle As UInt32) As FT_STATUS
        End Function

        Public Declare Function FT_OpenByIndex Lib "C:\Windows\system32\ftd2xx.dll" Alias "FT_Open" (ByVal intDeviceNumber As UInt32, ByRef lngHandle As UInt32) As FT_STATUS
        Public Declare Function FT_OpenBySerialNumber Lib "C:\Windows\system32\ftd2xx.dll" Alias "FT_OpenEx" (ByVal SerialNumber As String, ByVal lngFlags As UInt32, ByRef lngHandle As UInt32) As FT_STATUS
        Public Declare Function FT_OpenByDescription Lib "C:\Windows\system32\ftd2xx.dll" Alias "FT_OpenEx" (ByVal Description As String, ByVal lngFlags As UInt32, ByRef lngHandle As UInt32) As FT_STATUS

        <DllImport("FTD2XX.dll")> _
        Shared Function FT_Close(ByVal ftHandle As UInt32) As FT_STATUS
        End Function

        <DllImport("FTD2XX.dll")> _
        Shared Function FT_ResetDevice(ByVal ftHandle As UInt32) As FT_STATUS
        End Function

        <DllImport("FTD2XX.dll")> _
        Shared Function FT_CyclePort(ByVal ftHandle As UInt32) As FT_STATUS
        End Function

#End Region

#Region "Device Information"
        ' Import some of the FTDI D2XX.DLL functions with C# compatible parameters
        ' Create a C# friendly port for each function

        <DllImport("FTD2XX.dll")> _
        Shared Function FT_CreateDeviceInfoList(ByRef NumDevs As UInt32) As FT_STATUS
        End Function

        Public Declare Function FT_GetDeviceInfo Lib "C:\Windows\system32\ftd2xx.dll" (ByVal lngHandle As UInt32, ByRef lngFT_Type As UInt32, ByRef lngID As UInt32, ByVal pucSerialNumber As String, ByVal pucDescription As String, ByRef pvDummy As Byte) As FT_STATUS

        Public Declare Function FT_GetDeviceInfoDetail Lib "FTD2XX.dll" (ByVal Index As UInt32, ByRef lpdwFlags As UInt32, ByRef lpdwType As UInt32, ByRef lpdwID As UInt32, ByRef lpdwLocId As UInt32, ByVal SerialNumber As String, ByVal Description As String, ByRef ftHandle As UInt32) As FT_STATUS


#End Region

#Region "EEPROM"
        'EE_Read - not used since EE_ReadEx does the same and is more C# friendly
        'EE_Program - not used since EE_ProgramEx does the same and is more C# friendly

        'EE_ReadEx
        <DllImport("FTD2XX.dll")> _
        Shared Function FT_EE_ReadEx(ByVal ftHandle As UInt32, ByRef lpData As ft_program_data_struct, <MarshalAs(UnmanagedType.LPArray)> ByVal Manufacturer As Byte(), <MarshalAs(UnmanagedType.LPArray)> ByVal Manufacturerid As Byte(), <MarshalAs(UnmanagedType.LPArray)> ByVal Description As Byte(), <MarshalAs(UnmanagedType.LPArray)> ByVal SerialNumber As Byte()) As FT_STATUS
        End Function

        'EE_ProgramEx - TODO - test
        <DllImport("FTD2XX.dll")> _
        Shared Function FT_EE_ProgramEx(ByVal ftHandle As FT_HANDLE, ByRef lpData As ft_program_data_struct, <MarshalAs(UnmanagedType.LPArray)> ByVal Manufacturer As Byte(), <MarshalAs(UnmanagedType.LPArray)> ByVal Manufacturerid As Byte(), <MarshalAs(UnmanagedType.LPArray)> ByVal Description As Byte(), <MarshalAs(UnmanagedType.LPArray)> ByVal SerialNumber As Byte()) As FT_STATUS
        End Function

        'EraseEE - NOT IMPLEMENTED SINCE IT COULD SET VID TO 0 AND KILL THE DEVICE

        'EE_UARead
        <DllImport("FTD2XX.dll")> _
        Shared Function FT_EE_UARead(ByVal ftHandle As UInt32, <Out()> ByVal EEUAData As Char(), ByVal DataLen As UInt32, ByRef BytesRead As UInt32) As FT_STATUS
        End Function

        'EE_UAWrite
        <DllImport("FTD2XX.dll")> _
        Shared Function FT_EE_UAWrite(ByVal ftHandle As UInt32, ByVal EEUAData As Char(), ByVal DataLen As UInt32) As FT_STATUS
        End Function

        'EE_UASize
        <DllImport("FTD2XX.dll")> _
        Shared Function FT_EE_UASize(ByVal ftHandle As UInt32, ByRef EEUA_Size As UInt32) As FT_STATUS
        End Function

#End Region

#Region "Bit Banging"
        ' GetBitMode
        <DllImport("FTD2XX.dll")> _
        Shared Function FT_GetBitMode(ByVal ftHandle As FT_HANDLE, ByRef pucMode As Byte) As FT_STATUS
        End Function

        ' SetBitMode
        <DllImport("FTD2XX.dll")> _
        Shared Function FT_SetBitMode(ByVal ftHandle As FT_HANDLE, ByVal ucMask As Byte, ByVal ucEnable As Byte) As FT_STATUS
        End Function
#End Region

#Region "Read and write data"

        'Read
        <DllImport("FTD2XX.dll")> _
        Shared Function FT_Read(ByVal ftHandle As FT_HANDLE, ByRef lpBuffer As Byte, ByVal dwBytesToRead As UInt32, ByRef lpdwBytesReturned As UInt32) As FT_STATUS
        End Function

        'Write
        <DllImport("FTD2XX.dll")> _
        Shared Function FT_Write(ByVal ftHandle As FT_HANDLE, ByRef lpBuffer As Byte, ByVal dwBytesToWrite As UInt32, ByRef lpdwBytesWritten As UInt32) As FT_STATUS
        End Function

#End Region

#Region "Set Serial properties"

        ' SetBaudRate
        <DllImport("FTD2XX.dll")> _
        Shared Function FT_SetBaudRate(ByVal ftHandle As UInt32, ByVal dwBaudRate As UInt32) As FT_STATUS
        End Function

        ' SetDataCharacteristics
        <DllImport("FTD2XX.dll")> _
        Shared Function FT_SetDataCharacteristics(ByVal ftHandle As FT_HANDLE, ByVal uWordLength As Byte, ByVal uStopBits As Byte, ByVal uParity As Byte) As FT_STATUS
        End Function

        ' SetFlowControl
        <DllImport("FTD2XX.dll")> _
        Shared Function FT_SetFlowControl(ByVal ftHandle As FT_HANDLE, ByVal usFlowControl As Char, ByVal uXon As Byte, ByVal uXoff As Byte) As FT_STATUS
        End Function

        ' SetDtr
        <DllImport("FTD2XX.dll")> _
        Shared Function FT_SetDtr(ByVal ftHandle As FT_HANDLE) As FT_STATUS
        End Function

        ' ClrDtr
        <DllImport("FTD2XX.dll")> _
        Shared Function FT_ClrDtr(ByVal ftHandle As FT_HANDLE) As FT_STATUS
        End Function

        ' SetRts      
        <DllImport("FTD2XX.dll")> _
        Shared Function FT_SetRts(ByVal ftHandle As FT_HANDLE) As FT_STATUS
        End Function

        ' ClrRts
        <DllImport("FTD2XX.dll")> _
        Shared Function FT_ClrRts(ByVal ftHandle As FT_HANDLE) As FT_STATUS
        End Function

        ' GetModemStatus
        <DllImport("FTD2XX.dll")> _
        Shared Function FT_GetModemStatus(ByVal ftHandle As FT_HANDLE, ByRef lpdwModemStatus As UInt32) As FT_STATUS
        End Function

        ' SetChars
        <DllImport("FTD2XX.dll")> _
        Shared Function FT_SetChars(ByVal ftHandle As FT_HANDLE, ByVal uEventCh As Byte, ByVal uEventChEn As Byte, ByVal uErrorCh As Byte, ByVal uErrorChEn As Byte) As FT_STATUS
        End Function

        ' Purge
        <DllImport("FTD2XX.dll")> _
        Shared Function FT_Purge(ByVal ftHandle As FT_HANDLE, ByVal dwMask As UInt32) As FT_STATUS
        End Function

        ' SetTimeouts
        <DllImport("FTD2XX.dll")> _
        Shared Function FT_SetTimeouts(ByVal ftHandle As UInt32, ByVal dwReadTimeout As UInt32, ByVal dwWriteTimeout As UInt32) As FT_STATUS
        End Function

        ' GetQueueStatus
        <DllImport("FTD2XX.dll")> _
        Shared Function FT_GetQueueStatus(ByVal ftHandle As UInt32, ByRef lpdwAmountInRxQueue As UInt32) As FT_STATUS
        End Function

        ' GetStatus
        <DllImport("FTD2XX.dll")> _
        Shared Function FT_GetStatus(ByVal ftHandle As UInt32, ByRef lpdwAmountInRxQueue As UInt32, ByRef lpdwAmountInTxQueue As UInt32, ByRef lpdwEventStatus As UInt32) As FT_STATUS
        End Function

        ' SetBreakOn
        <DllImport("FTD2XX.dll")> _
        Shared Function FT_SetBreakOn(ByVal ftHandle As FT_HANDLE) As FT_STATUS
        End Function

        ' SetBreakOff
        <DllImport("FTD2XX.dll")> _
        Shared Function FT_SetBreakOff(ByVal ftHandle As FT_HANDLE) As FT_STATUS
        End Function

#End Region

#End Region

#Region "Data Accessors"

#Region "Device Information Accessors"

        ' Keep the serial number and description arrays private and expose the data as a 
        ' string. Since the D2XX dll requires that the serial number and the description
        ' be in byte arrays, we have to jump through some hoops for use with C#

        Private manufacturerArray As Byte() = New Byte(15) {}
        Public ReadOnly Property Manufacturer() As String
            Get
                Return manufacturerArray.ToString()
            End Get
        End Property

        Private manufacturerIDArray As Byte() = New Byte(15) {}
        Public ReadOnly Property ManufacturerID() As String
            Get
                Return (manufacturerIDArray.ToString())
            End Get
        End Property

        Private serialNumberArray As Byte() = New Byte(15) {}
        Public SerialNumber As String

        Public Description As String
        Private descriptionArray As Byte() = New Byte(63) {}

        ' the rest of the accessors are for UInt32
        Private index_p As UInt32
        Public Property Index() As UInt32
            Get
                Return index_p
            End Get
            Set(ByVal value As UInt32)
                index_p = value
            End Set
        End Property
        Private flags_p As UInt32
        Public Property Flags() As UInt32
            Get
                Return flags_p
            End Get
            Set(ByVal value As UInt32)
                flags_p = value
            End Set
        End Property
        Private type_p As UInt32
        Public Property Type() As UInt32
            Get
                Return type_p
            End Get
            Set(ByVal value As UInt32)
                type_p = value
            End Set
        End Property
        Private devicetype_p As UInt32
        Public Property DeviceType() As UInt32
            Get
                Return devicetype_p
            End Get
            Set(ByVal value As UInt32)
                devicetype_p = value
            End Set
        End Property
        Private locID_p As UInt32
        Public Property LocID() As UInt32
            Get
                Return locID_p
            End Get
            Set(ByVal value As UInt32)
                locID_p = value
            End Set
        End Property
        Private handle_p As UInt32
        Public Property Handle() As UInt32
            Get
                Return handle_p
            End Get
            Set(ByVal value As UInt32)
                handle_p = value
            End Set
        End Property
        Private numDevs_p As UInt32
        Public Property NumDevs() As UInt32
            Get
                Return numDevs_p
            End Get
            Set(ByVal value As UInt32)
                numDevs_p = value
            End Set
        End Property
        'private UInt32 devicenumber;
        'public UInt32 DeviceNumber { get { return devicenumber; } set { devicenumber = value; } }

#End Region

#Region "ft_program_data structure"
        '
        ' structure to hold program data for FT_Program function
        '
        Public Structure ft_program_data_struct

            Public Signature1 As UInt32 ' Header - must be 0x00000000
            Public Signature2 As UInt32 ' Header - must be 0xffffffff
            Public Version As UInt32 ' Header - FT_PROGRAM_DATA version
            '          0 = original
            '          1 = FT2232C extensions
            '			2 = FT232R extensions

            Public VendorId As UInt16 ' 0x0403
            Public ProductId As UInt16 ' 0x6001

            ' Manufacturer, ManufactureID, Description, and SerialNumber are changed
            ' to int to act as dummy variables for using this structure with the EEPROM
            ' read function - to simplify using C# we use only ReadEx so that we can
            ' get these variables as parameters and not as members of this structure
            'public char* Manufacturer;			// "FTDI"
            'public char* ManufacturerId;		// "FT"
            'public char* Description;			// "USB HS Serial Converter"
            'public char* SerialNumber;			// "FT000001" if fixed, or NULL
            Public Manufacturer As Integer ' "FTDI"
            Public ManufacturerId As Integer ' "FT"
            Public Description As Integer ' "USB HS Serial Converter"
            Public SerialNumber As Integer ' "FT000001" if fixed, or NULL
            Public MaxPower As UInt16 ' 0 < MaxPower <= 500
            Public PnP As UInt16 ' 0 = disabled, 1 = enabled
            Public SelfPowered As UInt16 ' 0 = bus powered, 1 = self powered
            Public RemoteWakeup As UInt16 ' 0 = not capable, 1 = capable
            '
            ' Rev4 extensions
            '
            Public Rev4 As Byte ' non-zero if Rev4 chip, zero otherwise
            Public IsoIn As Byte ' non-zero if in endpoint is isochronous
            Public IsoOut As Byte ' non-zero if out endpoint is isochronous
            Public PullDownEnable As Byte ' non-zero if pull down enabled
            Public SerNumEnable As Byte ' non-zero if serial number to be used
            Public USBVersionEnable As Byte ' non-zero if chip uses USBVersion
            Public USBVersion As UInt16 ' BCD (0x0200 => USB2)
            '
            ' FT2232C extensions
            '
            Private Rev5 As Byte ' non-zero if Rev5 chip, zero otherwise
            Private IsoInA As Byte ' non-zero if in endpoint is isochronous
            Private IsoInB As Byte ' non-zero if in endpoint is isochronous
            Private IsoOutA As Byte ' non-zero if out endpoint is isochronous
            Private IsoOutB As Byte ' non-zero if out endpoint is isochronous
            Private PullDownEnable5 As Byte ' non-zero if pull down enabled
            Private SerNumEnable5 As Byte ' non-zero if serial number to be used
            Private USBVersionEnable5 As Byte ' non-zero if chip uses USBVersion
            Private USBVersion5 As UInt16 ' BCD (0x0200 => USB2)
            Private AIsHighCurrent As Byte ' non-zero if interface is high current
            Private BIsHighCurrent As Byte ' non-zero if interface is high current
            Private IFAIsFifo As Byte ' non-zero if interface is 245 FIFO
            Private IFAIsFifoTar As Byte ' non-zero if interface is 245 FIFO CPU target
            Private IFAIsFastSer As Byte ' non-zero if interface is Fast serial
            Private AIsVCP As Byte ' non-zero if interface is to use VCP drivers
            Private IFBIsFifo As Byte ' non-zero if interface is 245 FIFO
            Private IFBIsFifoTar As Byte ' non-zero if interface is 245 FIFO CPU target
            Private IFBIsFastSer As Byte ' non-zero if interface is Fast serial
            Private BIsVCP As Byte ' non-zero if interface is to use VCP drivers
            '
            ' FT232R extensions
            '
            Public UseExtOsc As Byte ' Use External Oscillator
            Public HighDriveIOs As Byte ' High Drive I/Os
            Public EndpointSize As Byte ' Endpoint size

            Public PullDownEnableR As Byte ' non-zero if pull down enabled
            Public SerNumEnableR As Byte ' non-zero if serial number to be used

            Public InvertTXD As Byte ' non-zero if invert TXD
            Public InvertRXD As Byte ' non-zero if invert RXD
            Public InvertRTS As Byte ' non-zero if invert RTS
            Public InvertCTS As Byte ' non-zero if invert CTS
            Public InvertDTR As Byte ' non-zero if invert DTR
            Public InvertDSR As Byte ' non-zero if invert DSR
            Public InvertDCD As Byte ' non-zero if invert DCD
            Public InvertRI As Byte ' non-zero if invert RI

            Public Cbus0 As Byte ' Cbus Mux control
            Public Cbus1 As Byte ' Cbus Mux control
            Public Cbus2 As Byte ' Cbus Mux control
            Public Cbus3 As Byte ' Cbus Mux control
            Public Cbus4 As Byte ' Cbus Mux control

            Public RIsVCP As Byte ' non-zero if using VCP drivers

        End Structure

        ' create one instance of ft_program_data structure to expose properties
        Private FT_PROGRAM_DATA As ft_program_data_struct = New ft_program_data_struct()

#End Region

#Region "Data Accessors for FT_PROGRAM_DATA structure"
        Public Property VendorId() As UShort
            Get
                Return FT_PROGRAM_DATA.VendorId
            End Get
            Set(ByVal value As UShort)
                FT_PROGRAM_DATA.VendorId = value
            End Set
        End Property
        Public Property ProductId() As UShort
            Get
                Return FT_PROGRAM_DATA.ProductId
            End Get
            Set(ByVal value As UShort)
                FT_PROGRAM_DATA.ProductId = value
            End Set
        End Property
        Public Property SerNumEnable() As Byte
            Get
                Return FT_PROGRAM_DATA.SerNumEnable
            End Get
            Set(ByVal value As Byte)
                FT_PROGRAM_DATA.SerNumEnable = value
            End Set
        End Property
        Public Property PullDownEnabled() As Byte
            Get
                Return FT_PROGRAM_DATA.PullDownEnable
            End Get
            Set(ByVal value As Byte)
                FT_PROGRAM_DATA.PullDownEnable = value
            End Set
        End Property
        Public Property MaxPower() As UInt16
            Get
                Return FT_PROGRAM_DATA.MaxPower
            End Get
            Set(ByVal value As UInt16)
                FT_PROGRAM_DATA.MaxPower = value
            End Set
        End Property
        Public Property SelfPowered() As UInt16
            Get
                Return FT_PROGRAM_DATA.SelfPowered
            End Get
            Set(ByVal value As UInt16)
                FT_PROGRAM_DATA.SelfPowered = value
            End Set
        End Property
        Public Property USBVersion() As UInt16
            Get
                Return FT_PROGRAM_DATA.USBVersion
            End Get
            Set(ByVal value As UInt16)
                FT_PROGRAM_DATA.USBVersion = value
            End Set
        End Property
        Public Property RemoteWakeup() As UInt16
            Get
                Return FT_PROGRAM_DATA.RemoteWakeup
            End Get
            Set(ByVal value As UInt16)
                FT_PROGRAM_DATA.RemoteWakeup = value
            End Set
        End Property
        Public Property HighDriveIOs() As Byte
            Get
                Return FT_PROGRAM_DATA.HighDriveIOs
            End Get
            Set(ByVal value As Byte)
                FT_PROGRAM_DATA.HighDriveIOs = value
            End Set
        End Property
        Public Property RIsVCP() As Byte
            Get
                Return FT_PROGRAM_DATA.RIsVCP
            End Get
            Set(ByVal value As Byte)
                FT_PROGRAM_DATA.RIsVCP = value
            End Set
        End Property
        Public Property Cbus0() As Byte
            Get
                Return FT_PROGRAM_DATA.Cbus0
            End Get
            Set(ByVal value As Byte)
                FT_PROGRAM_DATA.Cbus0 = value
            End Set
        End Property
        Public Property Cbus1() As Byte
            Get
                Return FT_PROGRAM_DATA.Cbus1
            End Get
            Set(ByVal value As Byte)
                FT_PROGRAM_DATA.Cbus1 = value
            End Set
        End Property
        Public Property Cbus2() As Byte
            Get
                Return FT_PROGRAM_DATA.Cbus2
            End Get
            Set(ByVal value As Byte)
                FT_PROGRAM_DATA.Cbus2 = value
            End Set
        End Property
        Public Property Cbus3() As Byte
            Get
                Return FT_PROGRAM_DATA.Cbus3
            End Get
            Set(ByVal value As Byte)
                FT_PROGRAM_DATA.Cbus3 = value
            End Set
        End Property
        Public Property Cbus4() As Byte
            Get
                Return FT_PROGRAM_DATA.Cbus4
            End Get
            Set(ByVal value As Byte)
                FT_PROGRAM_DATA.Cbus4 = value
            End Set
        End Property
        Public Property InvertTXD() As Byte
            Get
                Return FT_PROGRAM_DATA.InvertTXD
            End Get
            Set(ByVal value As Byte)
                FT_PROGRAM_DATA.InvertTXD = value
            End Set
        End Property
        Public Property InvertRXD() As Byte
            Get
                Return FT_PROGRAM_DATA.InvertRXD
            End Get
            Set(ByVal value As Byte)
                FT_PROGRAM_DATA.InvertRXD = value
            End Set
        End Property
        Public Property InvertRTS() As Byte
            Get
                Return FT_PROGRAM_DATA.InvertRTS
            End Get
            Set(ByVal value As Byte)
                FT_PROGRAM_DATA.InvertRTS = value
            End Set
        End Property
        Public Property InvertCTS() As Byte
            Get
                Return FT_PROGRAM_DATA.InvertCTS
            End Get
            Set(ByVal value As Byte)
                FT_PROGRAM_DATA.InvertCTS = value
            End Set
        End Property
        Public Property InvertDTR() As Byte
            Get
                Return FT_PROGRAM_DATA.InvertDTR
            End Get
            Set(ByVal value As Byte)
                FT_PROGRAM_DATA.InvertDTR = value
            End Set
        End Property
        Public Property InvertDSR() As Byte
            Get
                Return FT_PROGRAM_DATA.InvertDSR
            End Get
            Set(ByVal value As Byte)
                FT_PROGRAM_DATA.InvertDSR = value
            End Set
        End Property
        Public Property InvertDCD() As Byte
            Get
                Return FT_PROGRAM_DATA.InvertDCD
            End Get
            Set(ByVal value As Byte)
                FT_PROGRAM_DATA.InvertDCD = value
            End Set
        End Property
        Public Property InvertRI() As Byte
            Get
                Return FT_PROGRAM_DATA.InvertRI
            End Get
            Set(ByVal value As Byte)
                FT_PROGRAM_DATA.InvertRI = value
            End Set
        End Property
#End Region

#End Region

#Region "friendly methods for DLL functions"

#Region "parse FT_STATUS"
        Public Function parseFT_STATUS(ByVal ftStaus As FT_STATUS) As String
            Dim temp As UInt32 = CUInt(ftStaus)

            Dim str As String = "NOT INITIALIZED"

            Select Case temp
                Case 0
                    str = "FT_OK"
                Case 1
                    str = "FT_INVALID_HANDLE"
                Case 2
                    str = "FT_DEVICE_NOT_FOUND"
                Case 3
                    str = "FT_DEVICE_NOT_OPENED"
                Case 4
                    str = "FT_IO_ERROR"
                Case 5
                    str = "FT_INSUFFICIENT_RESOURCES"
                Case 6
                    str = "FT_INVALID_PARAMETER"
                Case 7
                    str = "FT_INVALID_BAUD_RATE"
                Case 8
                    str = "FT_DEVICE_NOT_OPENED_FOR_ERASE"
                Case 9
                    str = "FT_DEVICE_NOT_OPENED_FOR_WRITE"
                Case 10
                    str = "FT_FAILED_TO_WRITE_DEVICE"
                Case 11
                    str = "FT_EEPROM_READ_FAILED"
                Case 12
                    str = "FT_EEPROM_WRITE_FAILED"
                Case 13
                    str = "FT_EEPROM_ERASE_FAILED"
                Case 14
                    str = "FT_EEPROM_NOT_PRESENT"
                Case 15
                    str = "FT_EEPROM_NOT_PROGRAMMED"
                Case 16
                    str = "FT_INVALID_ARGS"
                Case 17
                    str = "FT_OTHER_ERROR"
                Case Else
                    str = "NUMBER NOT IN FTSTATUS ENUM"
            End Select
            Return str
        End Function
#End Region

#Region "Support for the device open, close, and reset functions"

        ' 1. Opens a device based on a number that can be used in sequence to open
        '  multiple devices until. This number is NOT known as tied to a specific device.
        ' 2. Opens a device based on the serial number.
        ' 3. Opens a device based on the descriptor.
        ' 4. Opens a device based on the location ID.
        Public Function openDevice(ByVal device As UInt32, ByRef handle As UInt32) As FT_STATUS
            Return FT_Open(device, handle)
        End Function

        Public Function openSerialNumber(ByVal serialNumber As String, ByRef handle As UInt32) As FT_STATUS
            ' FT_OPEN_BY_SERIAL_NUMBER = 1
            Return FT_OpenBySerialNumber(serialNumber, 1, handle)
        End Function

        Public Function openDescription(ByVal description As String, ByRef handle As UInt32) As FT_STATUS
            ' FT_OPEN_BY_DESCRIPTION = 2
            Return FT_OpenByDescription(description, 2, handle)
        End Function

        Public Function openLocationID(ByVal locID As UInt32, ByRef handle As UInt32) As FT_STATUS
            ' FT_OPEN_BY_LOCATION = 4 
            Return FT_OpenByIndex(locID, handle)
        End Function

        Public Function closeDevice(ByVal handle As UInt32) As FT_STATUS
            Return FT_Close(handle)
        End Function

        Public Function resetDevice(ByVal handle As UInt32) As FT_STATUS
            Return FT_ResetDevice(handle)
        End Function

        Public Function cyclePort(ByVal handle As UInt32) As FT_STATUS
            Return FT_CyclePort(handle)
        End Function
#End Region

#Region "device information retrieval methods"

        ' friendly port of FT_CreateDeviceInfoList
        ' loads the number of devices into numDevs_p which is available
        ' through the NumDevs accessor. 
        Public Function createDeviceInfoList() As FT_STATUS
            Return FT_CreateDeviceInfoList(numDevs_p)
        End Function


        Public Function getDeviceInfoDetail(ByVal index_p As UInt32) As FT_STATUS
            Dim ftstatus As FT_STATUS

            Dim TempSerialNumber As String
            Dim TempDescription As String

            TempSerialNumber = Space(16)
            TempDescription = Space(64)

            ftstatus = FT_GetDeviceInfoDetail(index_p, flags_p, type_p, devicetype_p, locID_p, TempSerialNumber, TempDescription, handle_p)

            SerialNumber = Left(TempSerialNumber, InStr(1, TempSerialNumber, vbNullChar) - 1)
            Description = Left(TempDescription, InStr(1, TempDescription, vbNullChar) - 1)

            Return ftstatus
        End Function


        ' friendly port of FT_GetDeviceInfo for device with opened handle
        Public Function getDeviceInfo(ByVal handle As UInt32) As FT_STATUS
            Dim dummy As Byte = 0
            Return FT_GetDeviceInfo(handle, type_p, devicetype_p, SerialNumber, Description, dummy)
        End Function

        ' Helper function to return a string for the device type
        ' and since we are only using FT232R, we don't need this
        ' but are leaving it just in case...
        Public Function deviceTypeStr(ByVal typeNum As UInteger) As String
            'based on D2XX FT_DEVICE definitions
            Dim type As String() = {"FT_DEVICE_BM", "FT_DEVICE_AM", "FT_DEVICE_100AX", "FT_DEVICE_UNKNOWN", "FT_DEVICE_2232C", "FT_DEVICE_232R"}

            If typeNum < type.Length Then
                Return type(typeNum)
            Else
                MessageBox.Show("SM_D2XX Error in DeviceType(): typeNum is too long")
                Return "SM_D2XX Error in DeviceType(): typeNum is too long"
            End If

        End Function

#End Region

#Region "EEPROM programming"

        ' EEPROM Programming Interface
        ' The EEPROM holds 128 bytes of data, part of it is dedicated to the FTDI chip and the remaining
        ' space is available for the user to store data. The space available in the user area depends on
        ' the chip being used and available space can be determined with the EE_UASize function.

#Region "Read EEPROM"

        ' Read the EEPROM data into the EEPROM data structure.
        ' Read Manufacturer, ManufactureID, Description, SerialNumber into private variables
        ' Data accessors access these values thru the ft_program_data structure and the variables
        Public Sub readEEPROM(ByVal handle As UInt32)
            ' read the EEPROM and show data
            Dim ftStatus As FT_STATUS

            ftStatus = FT_EE_ReadEx(handle, FT_PROGRAM_DATA, manufacturerArray, manufacturerIDArray, descriptionArray, serialNumberArray)

            If ftStatus <> FT_STATUS.FT_OK Then
                closeDevice(handle)
                MessageBox.Show("Error: SM_FT232R_ReadEEPROM() FT_ReadEx() status !ok")
                Return
            End If

            closeDevice(handle)
        End Sub
#End Region

#Region "Program EEPROM"

        ' This function prevents the user from writing a 0 to the VID which will permanently kill a FT232R
        ' at least for Windows which will not open a device with VID = 0.
        Public Sub programEEPROM(ByVal handle As UInt32)
            Dim ftStatus As FT_STATUS

            FT_PROGRAM_DATA.Signature1 = &H0 ' Header - must be 0x00000000
            FT_PROGRAM_DATA.Signature2 = &HFFFFFFFFL ' Header - must be 0xffffffff
            FT_PROGRAM_DATA.Version = 2 ' Header - FT_PROGRAM_DATA version

            ' One last safety factor. Do not change the VID unless
            ' you know what you are doing - if you try to set it to 0
            ' this trap sets it to the FTDI vendor ID
            If FT_PROGRAM_DATA.VendorId = 0 Then
                FT_PROGRAM_DATA.VendorId = &H403 '0x403 FTDI default
                MessageBox.Show("You tried to set the VendorID to 0 which would have killed your device - so this action was not allowed and the device was set to 0x403, FTDI's VendorId, instead.")
            End If

            ftStatus = FT_EE_ProgramEx(handle, FT_PROGRAM_DATA, manufacturerArray, manufacturerIDArray, descriptionArray, serialNumberArray)
            If ftStatus <> FT_STATUS.FT_OK Then
                closeDevice(handle)
                MessageBox.Show("Error: SM_ProgramEx FT_EE_ProgramEx() status: " & ftStatus.ToString())
                Return
            End If

            closeDevice(handle)
        End Sub


#End Region

#Region "EEPROM User Area"

        Public Function readEEPROM_UA(ByVal handle As UInt32, <Out()> ByVal EEUAData As Char(), ByVal DataLen As UInt32, ByRef BytesRead As UInt32) As FT_STATUS
            Return FT_EE_UARead(handle, EEUAData, DataLen, BytesRead)
        End Function

        Public Function writeEEPROM_UA(ByVal handle As UInt32, <Out()> ByVal EEUAData As Char(), ByVal DataLen As UInt32) As FT_STATUS
            Return FT_EE_UAWrite(handle, EEUAData, DataLen)
        End Function

        Public Function getSizeEEPROM_UA(ByVal handle As UInt32, ByRef EEUA_Size As UInt32) As FT_STATUS
            Return FT_EE_UASize(handle, EEUA_Size)
        End Function

        Public Sub readEEPROM_UA(ByRef EEUA_Data As Char())
            Dim BytesRead As UInt32 = 0
            Dim handle As UInt32 = 0

            Dim ftStatus As FT_STATUS

            Dim EEUASize As UInteger = getEEUASize()

            ftStatus = openDevice(0, handle)

            If ftStatus <> FT_STATUS.FT_OK Then
                MessageBox.Show("Error: getEEUAData() FT_Open failure.")
                Return
            End If

            ftStatus = readEEPROM_UA(handle, EEUA_Data, EEUASize, BytesRead)
            If ftStatus <> FT_STATUS.FT_OK Then
                MessageBox.Show("Error: readEEUAData, FT_STATUS = " & ftStatus.ToString())
            End If

            If BytesRead <> EEUASize Then
                MessageBox.Show("Error: getEEUAData() - BytesRead != EEUZize")
            End If

            closeDevice(handle)
        End Sub

        Public Function writeEEPROM_UA(ByRef EEUA_Data As Char()) As Boolean
            Dim handle As UInt32 = 0

            Dim EEUASize As UInteger = getEEUASize()

            If EEUASize > 0 Then
                ' read the EEPROM and show data
                Dim ftStatus As FT_STATUS

                ftStatus = openDevice(0, handle)
                If ftStatus <> FT_STATUS.FT_OK Then
                    closeDevice(handle)
                    MessageBox.Show("Error: writeEEUAData() FT_Open failure.")
                    Return False
                End If

                ftStatus = writeEEPROM_UA(handle, EEUA_Data, EEUASize)

                If ftStatus <> FT_STATUS.FT_OK Then
                    closeDevice(handle)
                    MessageBox.Show("Error: writeEEUAData() FT_EE_UAWrite failure.")
                    Return False
                End If

                closeDevice(handle)
                Return True
            Else
                MessageBox.Show("Error: writeEEUASize() EEUASize is not greater than 0")
                Return False
            End If
        End Function

        Public Function getEEUASize() As UInt32
            Dim EEUA_Size As UInt32 = 0
            Dim handle As UInt32 = 0
            Dim ftStatus As FT_STATUS

            ftStatus = openDevice(0, handle)

            If ftStatus <> FT_STATUS.FT_OK Then
                closeDevice(handle)
                MessageBox.Show("Error: getEEUASize() FT_Open failure.")
            End If

            ftStatus = getSizeEEPROM_UA(handle, EEUA_Size)

            If ftStatus <> FT_STATUS.FT_OK Then
                closeDevice(handle)
                MessageBox.Show("Error: getEEUASize() FT_UASize failure.")
            End If

            closeDevice(handle)

            Return EEUA_Size
        End Function
#End Region

#Region "Bit Banging"

        ' GetBitMode
        Public Function bitBang_GetBitMode(ByVal handle As UInt32, ByRef Mode As Byte) As FT_STATUS
            Return FT_GetBitMode(handle, Mode)
        End Function

        ' SetBitMode
        Public Function bitBang_SetBitMode(ByVal handle As UInt32, ByVal Mask As Byte, ByVal Enable As Byte) As FT_STATUS
            Return FT_SetBitMode(handle, Mask, Enable)
        End Function


#End Region

#End Region

#Region "Serial functions for Windows Serial COM Port 'RS232'"


#Region "Read and write data"

        'Read
        Public Function read(ByVal handle As UInt32, ByRef Buffer As Byte, ByVal BytesToRead As UInt32, ByRef BytesReturned As UInt32) As FT_STATUS
            Return FT_Read(handle, Buffer, BytesToRead, BytesReturned)
        End Function

        'Write
        Public Function write(ByVal handle As UInt32, ByRef Buffer As Byte, ByVal BytesToWrite As UInt32, ByRef BytesWritten As UInt32) As FT_STATUS
            Return FT_Read(handle, Buffer, BytesToWrite, BytesWritten)
        End Function

#End Region

#Region "Set Serial properties"

        ' SetBaudRate
        Public Function setBaudRate(ByVal handle As UInt32, ByVal BaudRate As UInt32) As FT_STATUS
            Return FT_SetBaudRate(handle, BaudRate)
        End Function

        ' SetDataCharacteristics
        Public Function setDataCharacteristics(ByVal handle As UInt32, ByVal WordLength As Byte, ByVal StopBits As Byte, ByVal Parity As Byte) As FT_STATUS
            Return FT_SetDataCharacteristics(handle, WordLength, StopBits, Parity)
        End Function

        ' SetFlowControl
        Public Function setFlowControl(ByVal handle As FT_HANDLE, ByVal FlowControl As Char, ByVal Xon As Byte, ByVal Xoff As Byte) As FT_STATUS
            Return FT_SetFlowControl(handle, FlowControl, Xon, Xoff)
        End Function

        ' SetDtr
        Public Function setDtr(ByVal handle As UInt32) As FT_STATUS
            Return FT_SetDtr(handle)
        End Function

        ' ClrDtr
        Public Function clrDtr(ByVal handle As UInt32) As FT_STATUS
            Return FT_ClrDtr(handle)
        End Function

        ' SetRts      
        Public Function setRts(ByVal handle As UInt32) As FT_STATUS
            Return FT_SetRts(handle)
        End Function

        ' ClrRts
        Public Function clrRts(ByVal handle As UInt32) As FT_STATUS
            Return FT_ClrRts(handle)
        End Function

        ' GetModemStatus
        Public Function getModeStatus(ByVal handle As UInt32, ByRef ModemStatus As UInt32) As FT_STATUS
            Return FT_GetModemStatus(handle, ModemStatus)
        End Function

#End Region

#Region "Serial Miscellaneous"
        ' SetChars
        Public Function setChars(ByVal handle As FT_HANDLE, ByVal EventCh As Byte, ByVal EventChEn As Byte, ByVal ErrorCh As Byte, ByVal ErrorChEn As Byte) As FT_STATUS
            Return FT_SetChars(handle, EventCh, EventChEn, ErrorCh, ErrorChEn)
        End Function

        ' Purge
        Public Function purge(ByVal handle As UInt32, ByVal Mask As UInt32) As FT_STATUS
            Return FT_Purge(handle, Mask)
        End Function

        ' SetTimeouts
        Public Function setTimeouts(ByVal handle As UInt32, ByVal ReadTimeout As UInt32, ByVal WriteTimeout As UInt32) As FT_STATUS
            Return FT_SetTimeouts(handle, ReadTimeout, WriteTimeout)
        End Function

        ' GetQueueStatus
        Public Function getQueueStatus(ByVal handle As UInt32, ByRef AmountInRxQueue As UInt32) As FT_STATUS
            Return FT_GetQueueStatus(handle, AmountInRxQueue)
        End Function

        ' GetStatus
        Public Function getStatus(ByVal handle As UInt32, ByRef AmountInRxQueue As UInt32, ByRef AmountInTxQueue As UInt32, ByRef EventStatus As UInt32) As FT_STATUS
            Return FT_GetStatus(handle, AmountInRxQueue, AmountInTxQueue, EventStatus)
        End Function

        ' SetBreakOn
        Public Function setBreakOn(ByVal handle As UInt32) As FT_STATUS
            Return FT_SetBreakOn(handle)
        End Function

        ' SetBreakOff
        Public Function setBreakOff(ByVal handle As UInt32) As FT_STATUS
            Return FT_SetBreakOff(handle)
        End Function

#Region "SetEventNotification"

        'FT Notification Event Masks (See FT_SetEventNotification)
        Public Const FT_EVENT_RXCHAR As UInt32 = 1
        Public Const FT_EVENT_MODEM_STATUS As UInt32 = 2
        Public Const WAIT_ABANDONED As UInt32 = &H80
        Public Const WAIT_OBJECT_0 As UInt32 = 0
        Public Const WAIT_TIMEOUT As UInt32 = &H102
        Public Const INFINITE As UInt32 = &HFFFFFFFFL

        Public Function parseNotificationEvent(ByVal NotificationEvent As UInt32) As String
            Dim str As String = "NOT INITIALIZED"

            Select Case NotificationEvent
                Case FT_EVENT_RXCHAR
                    str = "FT_EVENT_RXCHAR"
                Case FT_EVENT_MODEM_STATUS
                    str = "FT_EVENT_MODEM_STATUS"
                Case WAIT_ABANDONED
                    str = "WAIT_ABANDONED"
                Case WAIT_OBJECT_0
                    str = "WAIT_OBJECT_0"
                Case WAIT_TIMEOUT
                    str = "WAIT_TIMEOUT"
                Case INFINITE
                    str = "INFINITE"
                Case Else
                    str = "INVALID NOTIFICATION EVENT NUMBER"
            End Select
            Return str
        End Function
#End Region

#End Region


#End Region

#End Region

    End Class
End Namespace

